@@ -4,9 +4,7 @@
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
 
-#ifdef HAVE_PCRE_LIB
 #include <pcre.h>               /* regular expression library */
-#endif
 #include "surf_routing_private.h"
 #include "surf/surf_routing.h"
 
@@ -65,9 +63,7 @@ typedef enum {
   SURF_MODEL_DIJKSTRA,
   SURF_MODEL_DIJKSTRACACHE,
   SURF_MODEL_NONE,
-#ifdef HAVE_PCRE_LIB
   SURF_MODEL_RULEBASED
-#endif
 } e_routing_types;
 
 struct s_model_type routing_models[] = { {"Full",
@@ -90,12 +86,10 @@ struct s_model_type routing_models[] = { {"Full",
  model_dijkstra_both_unload, model_dijkstra_both_end},
 {"none", "No routing (usable with Constant network only)",
  model_none_create, model_none_load, model_none_unload, model_none_end},
-#ifdef HAVE_PCRE_LIB
 {"RuleBased", "Rule-Based routing data (...)", model_rulebased_create,
  model_rulebased_load, model_rulebased_unload, model_rulebased_end},
 {"Vivaldi", "Vivaldi routing", model_rulebased_create,
   model_rulebased_load, model_rulebased_unload, model_rulebased_end},
-#endif
 {NULL, NULL, NULL, NULL, NULL, NULL}
 };
 
@@ -127,7 +121,8 @@ static double base_vivaldi_get_latency (const char *src, const char *dst)
 
   xbt_assert(euclidean_dist>=0, "Euclidean Dist is less than 0\"%s\" and \"%.2f\"", src, euclidean_dist);
 
-  return euclidean_dist;
+  //From .ms to .s
+  return euclidean_dist / 1000;
 }
 
 static double vivaldi_get_link_latency (routing_component_t rc,const char *src, const char *dst, route_extended_t e_route)
@@ -808,6 +803,27 @@ static xbt_dynar_t get_route(const char *src, const char *dst)
   return route;
 }
 
+/**
+ * \brief Generic method: find a route between hosts
+ *
+ * \param src the source host name
+ * \param dst the destination host name
+ *
+ * same as get_route, but return NULL if any exception is raised.
+ */
+static xbt_dynar_t get_route_or_null(const char *src, const char *dst)
+{
+  xbt_dynar_t route = NULL;
+  xbt_ex_t exception;
+  TRY {
+    get_route_latency(src, dst, &route, NULL, 1);
+  }CATCH(exception) {
+    xbt_ex_free(exception);
+    return NULL;
+  }
+  return route;
+}
+
 /**
  * \brief Generic method: find a route between hosts
  *
@@ -934,6 +950,7 @@ void routing_model_create(size_t size_of_links, void *loopback, double_f_cpvoid_
   global_routing = xbt_new0(s_routing_global_t, 1);
   global_routing->root = NULL;
   global_routing->get_route = get_route;
+  global_routing->get_route_or_null = get_route_or_null;
   global_routing->get_latency = get_latency;
   global_routing->get_route_no_cleanup = get_route_no_cleanup;
   global_routing->get_onelink_routes = get_onelink_routes;
@@ -1506,7 +1523,7 @@ void routing_parse_Scluster(void)
   char *cluster_availability_file = A_surfxml_cluster_availability_file;
   char *cluster_state_file = A_surfxml_cluster_state_file;
   char *host_id, *groups, *link_id = NULL;
-  char *router_id, *link_backbone;
+  char *router_id = xbt_strdup(A_surfxml_cluster_router_id);
   char *availability_file = xbt_strdup(cluster_availability_file);
   char *state_file = xbt_strdup(cluster_state_file);
 
@@ -1517,9 +1534,7 @@ void routing_parse_Scluster(void)
   xbt_dict_set(patterns,"prefix",cluster_prefix,NULL);
   xbt_dict_set(patterns,"suffix",cluster_suffix,NULL);
 
-#ifdef HAVE_PCRE_LIB
   char *route_src_dst;
-#endif
   unsigned int iter;
   int start, end, i;
   xbt_dynar_t radical_elements;
@@ -1527,12 +1542,6 @@ void routing_parse_Scluster(void)
   int cluster_sharing_policy = AX_surfxml_cluster_sharing_policy;
   int cluster_bb_sharing_policy = AX_surfxml_cluster_bb_sharing_policy;
 
-#ifndef HAVE_PCRE_LIB
-  xbt_dynar_t tab_elements_num = xbt_dynar_new(sizeof(int), NULL);
-  char *route_src, *route_dst;
-  int j;
-#endif
-
   static unsigned int surfxml_buffer_stack_stack_ptr = 1;
   static unsigned int surfxml_buffer_stack_stack[1024];
 
@@ -1541,13 +1550,8 @@ void routing_parse_Scluster(void)
   surfxml_bufferstack_push(1);
 
   SURFXML_BUFFER_SET(AS_id, cluster_id);
-#ifdef HAVE_PCRE_LIB
   SURFXML_BUFFER_SET(AS_routing, "RuleBased");
   XBT_DEBUG("<AS id=\"%s\"\trouting=\"RuleBased\">", cluster_id);
-#else
-  SURFXML_BUFFER_SET(AS_routing, "Full");
-  XBT_DEBUG("<AS id=\"%s\"\trouting=\"Full\">", cluster_id);
-#endif
   SURFXML_START_TAG(AS);
 
   radical_elements = xbt_str_split(cluster_radical, ",");
@@ -1558,9 +1562,6 @@ void routing_parse_Scluster(void)
       surf_parse_get_int(&start,
                          xbt_dynar_get_as(radical_ends, 0, char *));
       host_id = bprintf("%s%d%s", cluster_prefix, start, cluster_suffix);
-#ifndef HAVE_PCRE_LIB
-      xbt_dynar_push_as(tab_elements_num, int, start);
-#endif
       link_id = bprintf("%s_link_%d", cluster_id, start);
 
       xbt_dict_set(patterns, "radical", bprintf("%d", start), xbt_free);
@@ -1620,9 +1621,6 @@ void routing_parse_Scluster(void)
       surf_parse_get_int(&end, xbt_dynar_get_as(radical_ends, 1, char *));
       for (i = start; i <= end; i++) {
         host_id = bprintf("%s%d%s", cluster_prefix, i, cluster_suffix);
-#ifndef HAVE_PCRE_LIB
-        xbt_dynar_push_as(tab_elements_num, int, i);
-#endif
         link_id = bprintf("%s_link_%d", cluster_id, i);
 
         xbt_dict_set(patterns, "radical", bprintf("%d", i), xbt_free);
@@ -1685,11 +1683,8 @@ void routing_parse_Scluster(void)
   xbt_dynar_free(&radical_elements);
 
   XBT_DEBUG(" ");
-  router_id =
-      bprintf("%s%s_router%s", cluster_prefix, cluster_id,
-              cluster_suffix);
-  //link_router = bprintf("%s_link_%s_router", cluster_id, cluster_id);
-  link_backbone = bprintf("%s_backbone", cluster_id);
+  if(!strcmp(router_id,""))
+	  router_id = bprintf("%s%s_router%s", cluster_prefix, cluster_id, cluster_suffix);
 
   XBT_DEBUG("<router id=\"%s\"/>", router_id);
   SURFXML_BUFFER_SET(router_id, router_id);
@@ -1697,31 +1692,8 @@ void routing_parse_Scluster(void)
   SURFXML_START_TAG(router);
   SURFXML_END_TAG(router);
 
-  //TODO
-//  xbt_dict_set(patterns, "radical", xbt_strdup("_router"), xbt_free);
-//  temp_cluster_bw = xbt_strdup(cluster_bw);
-//  temp_cluster_bw = replace_random_parameter(temp_cluster_bw);
-//  temp_cluster_lat = xbt_strdup(cluster_lat);
-//  temp_cluster_lat = replace_random_parameter(temp_cluster_lat);
-//  XBT_DEBUG("<link\tid=\"%s\" bw=\"%s\" lat=\"%s\"/>", link_router,temp_cluster_bw, temp_cluster_lat);
-//  A_surfxml_link_state = A_surfxml_link_state_ON;
-//  A_surfxml_link_sharing_policy = A_surfxml_link_sharing_policy_SHARED;
-//  if(cluster_sharing_policy == A_surfxml_cluster_sharing_policy_FULLDUPLEX)
-//  {A_surfxml_link_sharing_policy =  A_surfxml_link_sharing_policy_FULLDUPLEX;}
-//  if(cluster_sharing_policy == A_surfxml_cluster_sharing_policy_FATPIPE)
-//  {A_surfxml_link_sharing_policy =  A_surfxml_link_sharing_policy_FATPIPE;}
-//  SURFXML_BUFFER_SET(link_id, link_router);
-//  SURFXML_BUFFER_SET(link_bandwidth, temp_cluster_bw);
-//  SURFXML_BUFFER_SET(link_latency, temp_cluster_lat);
-//  SURFXML_BUFFER_SET(link_bandwidth_file, "");
-//  SURFXML_BUFFER_SET(link_latency_file, "");
-//  SURFXML_BUFFER_SET(link_state_file, "");
-//  SURFXML_START_TAG(link);
-//  SURFXML_END_TAG(link);
-
-//  xbt_free(temp_cluster_bw);
-//  xbt_free(temp_cluster_lat);
-
+  if(strcmp(cluster_bb_bw,"") && strcmp(cluster_bb_lat,"")){
+  char *link_backbone = bprintf("%s_backbone", cluster_id);
   XBT_DEBUG("<link\tid=\"%s\" bw=\"%s\" lat=\"%s\"/>", link_backbone,cluster_bb_bw, cluster_bb_lat);
   A_surfxml_link_state = A_surfxml_link_state_ON;
   A_surfxml_link_sharing_policy = A_surfxml_link_sharing_policy_SHARED;
@@ -1735,10 +1707,11 @@ void routing_parse_Scluster(void)
   SURFXML_BUFFER_SET(link_state_file, "");
   SURFXML_START_TAG(link);
   SURFXML_END_TAG(link);
+  free(link_backbone);
+  }
 
   XBT_DEBUG(" ");
 
-#ifdef HAVE_PCRE_LIB
   char *new_suffix = xbt_strdup("");
 
   radical_elements = xbt_str_split(cluster_suffix, ".");
@@ -1765,11 +1738,13 @@ void routing_parse_Scluster(void)
   A_surfxml_route_symmetrical = A_surfxml_route_symmetrical_NO;
   SURFXML_START_TAG(route);
 
+  if(strcmp(cluster_bb_bw,"") && strcmp(cluster_bb_lat,"")){
   XBT_DEBUG("<link_ctn\tid=\"%s\"/>", pcre_link_backbone);
   SURFXML_BUFFER_SET(link_ctn_id, pcre_link_backbone);
   A_surfxml_link_ctn_direction = A_surfxml_link_ctn_direction_NONE;
   SURFXML_START_TAG(link_ctn);
   SURFXML_END_TAG(link_ctn);
+  }
 
   XBT_DEBUG("</route>");
   SURFXML_END_TAG(route);
@@ -1790,11 +1765,13 @@ void routing_parse_Scluster(void)
   SURFXML_START_TAG(link_ctn);
   SURFXML_END_TAG(link_ctn);
 
+  if(strcmp(cluster_bb_bw,"") && strcmp(cluster_bb_lat,"")){
   XBT_DEBUG("<link_ctn\tid=\"%s\"/>", pcre_link_backbone);
   SURFXML_BUFFER_SET(link_ctn_id, pcre_link_backbone);
   A_surfxml_link_ctn_direction = A_surfxml_link_ctn_direction_NONE;
   SURFXML_START_TAG(link_ctn);
   SURFXML_END_TAG(link_ctn);
+  }
 
   XBT_DEBUG("</route>");
   SURFXML_END_TAG(route);
@@ -1807,11 +1784,13 @@ void routing_parse_Scluster(void)
   A_surfxml_route_symmetrical = A_surfxml_route_symmetrical_NO;
   SURFXML_START_TAG(route);
 
+  if(strcmp(cluster_bb_bw,"") && strcmp(cluster_bb_lat,"")){
   XBT_DEBUG("<link_ctn\tid=\"%s\"/>", pcre_link_backbone);
   SURFXML_BUFFER_SET(link_ctn_id, pcre_link_backbone);
   A_surfxml_link_ctn_direction = A_surfxml_link_ctn_direction_NONE;
   SURFXML_START_TAG(link_ctn);
   SURFXML_END_TAG(link_ctn);
+  }
 
   XBT_DEBUG("<link_ctn\tid=\"%s\"/>", pcre_link_dst);
   SURFXML_BUFFER_SET(link_ctn_id, pcre_link_dst);
@@ -1840,11 +1819,13 @@ void routing_parse_Scluster(void)
   SURFXML_START_TAG(link_ctn);
   SURFXML_END_TAG(link_ctn);
 
+  if(strcmp(cluster_bb_bw,"") && strcmp(cluster_bb_lat,"")){
   XBT_DEBUG("<link_ctn\tid=\"%s\"/>", pcre_link_backbone);
   SURFXML_BUFFER_SET(link_ctn_id, pcre_link_backbone);
   A_surfxml_link_ctn_direction = A_surfxml_link_ctn_direction_NONE;
   SURFXML_START_TAG(link_ctn);
   SURFXML_END_TAG(link_ctn);
+  }
 
   XBT_DEBUG("<link_ctn\tid=\"%s\"/>", pcre_link_dst);
   SURFXML_BUFFER_SET(link_ctn_id, pcre_link_dst);
@@ -1861,79 +1842,8 @@ void routing_parse_Scluster(void)
   free(pcre_link_backbone);
   free(pcre_link_src);
   free(route_src_dst);
-#else
-  for (i = 0; i <= xbt_dynar_length(tab_elements_num); i++) {
-    for (j = 0; j <= xbt_dynar_length(tab_elements_num); j++) {
-      if (i == xbt_dynar_length(tab_elements_num)) {
-        route_src = router_id;
-      } else {
-        route_src =
-            bprintf("%s%d%s", cluster_prefix,
-                    xbt_dynar_get_as(tab_elements_num, i, int),
-                    cluster_suffix);
-      }
-
-      if (j == xbt_dynar_length(tab_elements_num)) {
-        route_dst = router_id;
-      } else {
-        route_dst =
-            bprintf("%s%d%s", cluster_prefix,
-                    xbt_dynar_get_as(tab_elements_num, j, int),
-                    cluster_suffix);
-      }
-
-      XBT_DEBUG("<route\tsrc=\"%s\"\tdst=\"%s\"", route_src, route_dst);
-      XBT_DEBUG("symmetrical=\"NO\">");
-      SURFXML_BUFFER_SET(route_src, route_src);
-      SURFXML_BUFFER_SET(route_dst, route_dst);
-      A_surfxml_route_symmetrical = A_surfxml_route_symmetrical_NO;
-      SURFXML_START_TAG(route);
-
-      if (i != xbt_dynar_length(tab_elements_num)){
-          route_src =
-                bprintf("%s_link_%d", cluster_id,
-                        xbt_dynar_get_as(tab_elements_num, i, int));
-		  XBT_DEBUG("<link_ctn\tid=\"%s\"/>", route_src);
-		  SURFXML_BUFFER_SET(link_ctn_id, route_src);
-		  A_surfxml_link_ctn_direction = A_surfxml_link_ctn_direction_NONE;
-		  if(cluster_sharing_policy == A_surfxml_cluster_sharing_policy_FULLDUPLEX)
-		  {A_surfxml_link_ctn_direction = A_surfxml_link_ctn_direction_UP;}
-		  SURFXML_START_TAG(link_ctn);
-		  SURFXML_END_TAG(link_ctn);
-		  free(route_src);
-      }
-
-      XBT_DEBUG("<link_ctn\tid=\"%s_backbone\"/>", cluster_id);
-      SURFXML_BUFFER_SET(link_ctn_id, bprintf("%s_backbone", cluster_id));
-      A_surfxml_link_ctn_direction = A_surfxml_link_ctn_direction_NONE;
-      SURFXML_START_TAG(link_ctn);
-      SURFXML_END_TAG(link_ctn);
-
-      if (j != xbt_dynar_length(tab_elements_num)) {
-          route_dst =
-                bprintf("%s_link_%d", cluster_id,
-                        xbt_dynar_get_as(tab_elements_num, j, int));
-		  XBT_DEBUG("<link_ctn\tid=\"%s\"/>", route_dst);
-		  SURFXML_BUFFER_SET(link_ctn_id, route_dst);
-		  A_surfxml_link_ctn_direction = A_surfxml_link_ctn_direction_NONE;
-		  if(cluster_sharing_policy == A_surfxml_cluster_sharing_policy_FULLDUPLEX)
-		  {A_surfxml_link_ctn_direction = A_surfxml_link_ctn_direction_DOWN;}
-		  SURFXML_START_TAG(link_ctn);
-		  SURFXML_END_TAG(link_ctn);
-		  free(route_dst);
-      }
-
-      XBT_DEBUG("</route>");
-      SURFXML_END_TAG(route);
-    }
-  }
-  xbt_dynar_free(&tab_elements_num);
-
-#endif
 
   free(router_id);
-  free(link_backbone);
-  //free(link_router);
   xbt_dict_free(&patterns);
   free(availability_file);
   free(state_file);
